


/*          3Pi Command Processor for RobotBASIC with full sensor support
 *         Copyright January 2010 by John Blankenship, all rights reserved       
 * 
 * This code allows the 3Pi to communicate with RobotBASIC using the built-in 
 * communication protocol.  The sensors supported include 5 line sensors, 
 * 4 bumper sensors, 5 infrared perimeter sensors, 1 beacon detector (recognizes
 * multiple beacons), electronic compass, battery level, and distance measurement
 * using servo controlled turret.
 *
 * Care was taken to allow the STANDARD Pololu 3Pi library routines to be used for all sensors.
 * General information about the hardware modifications can be found at www.RobotBASIC.com
 * Full details are provided in the book "Hardware Interfacing with RobotBASIC'
 * A full complement of interesting projects are provided in the book "RobotBASIC
 * Projects for the Pololu".   Special commands have been added to provide special 
 * initialization and calibration capabilities.
 *
 * Note: Bumper sensors and Infared Perimeter sensors are multiplexed over the existing
 * line sensors, using the standard library routines.
 *
 * Suggested websites:
 *     http://www.RobotBASIC.com
 *     http://www.pololu.com
 *     http://forum.pololu.com
 *
 */

// The 3pi include file must be at the beginning of any program that
// uses the Pololu AVR library and 3pi.
#include <C:\Users\John\Documents\AVR\Include\pololu\3pi.h>
//#include <string.h>

#include <avr/IO.h>

// This include file allows data to be stored in program space.  The
// ATmega168 has 16k of program space compared to 1k of RAM, so large
// pieces of static data should be stored in program space.
#include <avr/pgmspace.h>


//====================================================================
// initialization of global variables and constants
//====================================================================

// A couple of simple tunes, stored in program space.
const char welcome[] PROGMEM = ">g32>>c32";
const char go[] PROGMEM = "L16 cdegreg4";

// New I/O pin definitions: Original standard 3Pi definitions still apply
  #define Servo PORTB0
  #define Beacon PORTB1
  #define CompassClock PORTD2
  #define CompassData PORTD4
  #define MuxCtl PORTD7

void CalibrateCompass(void);
int  ReadCompass(void);

// ================================================================
// Initialization routine
// Initializes the 3pi, displays a welcome message, calibrates, and
// plays the initial music.
// ================================================================
void initialize()
{
    // set up Direction Register when necessary for new pin assignments
       DDRB |= 1<<Servo;          // set pin to output for servo control
       DDRB &= ~(1<<Beacon);      // set pin to input to read beacon data
       DDRD |= 1<MuxCtl;          // set pin to output to control MUX circuits

    // initially set Servo and Mux outputs LOW
	   PORTB &= ~(1<Servo);
	   PORTD &= ~(1<MuxCtl);

    // Compass I/O setup is special as the compass is I2C so both ports must float
    // Initial data is set LOW so any time the PIN is an OUTPUT, it will be LOW
    // When a PIN floats, EXTERNAL pull-up's allow it to appear HIGH
	   DDRD |=  CompassClock;        // make both pins output
	   DDRD |=  CompassData;         // so we can set the DATA register
	   PORTD &= ~(1<<CompassClock);   // Set both outputs LOW
	   PORTD &= ~(1<<CompassData);
	   DDRD &=  ~(1<<CompassClock);   // Set both to INPUT so they FLOAT
	   DDRD &=  ~(1<<CompassData);

    // Note: The analog pins for reading the battery voltage and Distance value
	//       are already setup properly, as is PC5 which controls the infrared
	//       source for the line sensors (which must be turned OFF when reading
	//       the bumpers or the infrared parimeter sensors.  PD0 and PD1 are also
	//       properly set up for serial communication by the standard 3Pi initialization

//	unsigned int counter; // used as a simple timer
//	unsigned int sensors[5]; // an array to hold line sensor values (or other MUX data)
	char CR=13;
//char newRate[17]= "set baud 115200";
//newRate[15]=13;

	// This must be called at the beginning of 3pi code, to set up the
	// sensors.  Pololu uses a value of 2000 for the timeout, which
	// corresponds to 2000*0.4 us = 0.8 ms on our 20 MHz processor.
	pololu_3pi_init(2000);
	
	// Play welcome music
	play_from_program_space(welcome);
	delay_ms(1000);


    set_motors(0,0); // make sure motors are initially off
    serial_set_baud_rate(9600);
    delay_ms(200);
//serial_set_baud_rate(115200);
}

// ===================================================================
// Simple Bluetooth control module
// Assumes only one Bluetooth device is within range and installed
// Waits for commands from PC and returns sensor values
// as described in Robot Programmer's Bonanza by Blankenship/Mishal
// ===================================================================
void BlueToothControl(void)
{
   unsigned int sensor_values[5];
   unsigned char sv4;
   unsigned char sv3;
   unsigned char sv2;
   unsigned char sv1;
   unsigned char sv0;
   unsigned char svtotal;
   static unsigned char oldSensors=0;
   static unsigned char oldIR=0;
   static unsigned char oldBumpers=0;
   static unsigned char Rotate=1;
   unsigned char command;
   unsigned char argument;
   unsigned char incoming[2];
   static char speed=35; // originally 35
   static char SensorBuffer[5]={0b0,0b0,0b0,0b0,0b0};
   static char LastTime;
   static int Loffset=0;
   static int Roffset=0;
   static char RotTime=14;
   static char MoveTime=80;
   static char FastTurns=1;
   int i,battery,counter;
   unsigned int sum;
   unsigned int ave;
   unsigned char samples;
   incoming[0]=0;
   incoming[1]=0;
   serial_receive_blocking(incoming,2,400);
   command=incoming[0];
   argument=incoming[1];
   if (command==0)
     {
     set_motors(0,0);
     return;
     }
   SensorBuffer[3]=0;
   SensorBuffer[4]=0;
   switch (command)
   {
          // ======= Locate =========
		  case 3:
            set_motors(0,0);
			play_from_program_space(go);
		  break;

          // ====== Forward =========
		  case 6:
		    if (argument==1)
			  {
			  set_motors(60,60);// overcome friction etc
              delay_ms(15);
			  set_motors(speed+Loffset,speed+Roffset);
              }
            if (argument==0)
			  {
			  set_motors(0,0);
              }
        	if (argument>1)
			   {
			   set_motors(60,60);// overcome friction etc
               delay_ms(15);
			   set_motors(speed+Loffset,speed+Roffset);
			   delay_ms(MoveTime*argument);
			   set_motors(0,0);
			   }	 			
		  break;

          // ====== Reverse =========
		  case 7:
		    if (argument==1)
			  {
			  set_motors(-60,-60);// overcome friction etc
              delay_ms(15);
			  set_motors(-(speed+Loffset),-(speed+Roffset));
              }
            if (argument==0)
			  {
			  set_motors(0,0);
              }
        	if (argument>1)
			   {
			   set_motors(-60,-60);// overcome friction etc
               delay_ms(15);
			   set_motors(-(speed+Loffset),-(speed+Roffset));
			   delay_ms(MoveTime*argument);
			   set_motors(0,0);
			   }
		  break;

          // ======= Right  =========
		  case 12:
		    if (argument==1)
			  {  
			  set_motors(60,-60*FastTurns);// overcome friction etc
              delay_ms(15);
			  set_motors(20,-20*FastTurns); // originally 20
              }
            if (argument==0)
			  {
			  set_motors(0,0);
              }
        	if (argument>1)
			   {
			   set_motors(60,-60);// overcome friction etc
               delay_ms(15);   
			   set_motors(20,-20);
			   delay_ms(RotTime*argument);
			   set_motors(0,0);
			   }
		  break;

          // ======= Left ===========
		  case 13:
		    if (argument==1)
			  {
			  set_motors(-60*FastTurns,60);// overcome friction etc
              delay_ms(14);
			  set_motors(-20*FastTurns,20);
              }
            if (argument==0)
		  	  {
			  set_motors(0,0);
              }
        	if (argument>1)
			   {  
			   set_motors(-60,60);// overcome friction etc
               delay_ms(15);
			   set_motors(-20,20);
			   delay_ms(RotTime*argument);
			   set_motors(0,0);
			   }
		  break;

		  // ======= Speed ==========
		  case 36:
		    speed=argument;
		  break;

		  // ======= Compass =========
		  case 24:
		    sum=ReadCompass()/10;
            SensorBuffer[3]=sum>>8;
			SensorBuffer[4]=sum&255;
		  break;

		  // ======= Beacon ==========
		  case 96:
		    if (PINB&(1<<Beacon))
			  {
			  SensorBuffer[3]=0;
			  SensorBuffer[4]=1;
			  }
// use argument to test for proper beacon - current tests for ANY beacon
		  break;

		  // === Ranging Right (Distance) ===
		  case 192:
            set_analog_mode(MODE_8_BIT); // 8-bit analog-to-digital conversions
            sum = 0;
            ave = 0;     
            for(samples=0;samples<10;samples++)
              {
			  start_analog_conversion(TRIMPOT); // start conversion
              while (analog_is_converting())
			     ; // wait till conversion is done...
              sum += analog_conversion_result(); // get result
              }
            ave = sum / 10; // compute 10-sample average of ADC result
            SensorBuffer[3]=0;
			SensorBuffer[4]=ave;
// convert ave to a distance in inches
		  break;

		  // === Ranging  Left (Distance) ===
		  case 193:
// testing servo
            for(i=0;i<30;i++) // do enough pulses to ensure movement
			  {
			  PORTB |= (1<<Servo);  // pulse line high
			  delay_us(1400);
			  PORTB &= ~(1<<Servo); // return to low
			  delay_ms(1);
			  }
            SensorBuffer[3]=0;
			SensorBuffer[4]=0;
		  break;

		  // ======== Charge Level ==========
		  case 108:
		    battery=read_battery_millivolts();
			battery=battery/52;
			if (battery>100) battery=100;
			SensorBuffer[3]=battery>>8;
			SensorBuffer[4]=battery&255;
		  break;

          // The following commands are issued from RobotBASIC through the rCommand statement
		 
		  // ======= Play Sound =============
		  case 111:
		    if (argument==1) play_from_program_space(welcome);
			if (argument==2) play_from_program_space(go);
			if (argument==3) play_from_program_space("c");
			if (argument==4) play_from_program_space("c32");
		  break;

		  // ======= Calibrate Servo ========
		  case 112:
		  break;

		  // ==== Calibrate Line Sensors ====
		  case 113:
		    set_motors(0,0);
		    delay_ms(1000);
     	    // Auto-calibration: turn right and left while calibrating the sensors
	        for(counter=0;counter<80;counter++)
	          {
		      if(counter < 20 || counter >= 60)
			     set_motors(40,-40);
		      else
			     set_motors(-40,40);

		      // This function records a set of sensor readings and keeps
		      // track of the minimum and maximum values encountered.  The
		      // IR_EMITTERS_ON argument means that the IR LEDs will be
		      // turned on during the reading, which is usually what you want
		      calibrate_line_sensors(IR_EMITTERS_ON);

		      // Since our counter runs to 80, the total delay will be
		      // 80*20 = 1600 ms.
		      delay_ms(20);
	          }
		  break;

		  // ==== Calibrate Rotation Time ===
		    RotTime=argument;
		  case 114:
		  break;

		  // ====== Calibrate F/R Time ======
		  case 115:
		    MoveTime=argument;
		  break;

		  // ======= Calibrate Drift ========
		  case 116:
		    Roffset = argument&15;
			Loffset = argument>>4;
			if (Roffset>7) Roffset=16-Roffset;
			if (Loffset>7) Loffset=16-Loffset;
		  break;

		  // ====== Calibrate Compass =======
		  case 117:
		    CalibrateCompass();
		  break;

		  // ====== Enable Slow Turns =======
		  case 118:
		    FastTurns=argument;
		  break;

		  // ========== Halt ================
		  case 119:
		    set_motors(0,0);
          break;

		  // ======== Defaults ==============
		  case 120:
		    speed=35;
			RotTime=14;
			MoveTime=80;
			Loffset=0;
			Roffset=0;
			FastTurns=1;
		  break;
	 };

	 // now read the line sensors	 
     read_line_sensors_calibrated(sensor_values,IR_EMITTERS_ON);
	 if(sensor_values[4] > 200)
	   sv4 = 16;
     else
	   sv4 = 0;
     if(sensor_values[3] > 200)
       sv3 = 1;
     else
       sv3 = 0;

     if (sensor_values[2] > 200)
       sv2 = 2;
     else
       sv2 = 0;

    if (sensor_values[1] > 200)
       sv1 = 4;
    else
       sv1 = 0;

    if (sensor_values[0] >200)
	   sv0 = 8;
    else
	   sv0 = 0;

    svtotal = sv4+sv3+sv2+sv1+sv0;
	// if sensor values have changed... stop motors
	if (oldSensors != svtotal)
	  {
	   set_motors(0,0);  
	  }
    oldSensors = svtotal;  // remember current sensors
	SensorBuffer[2]=svtotal; // prepare to send sensors back

	 // now read the IR sensors	 
	 PORTD &= ~(1<<MuxCtl);  // select IR sensors
     read_line_sensors_calibrated(sensor_values,IR_EMITTERS_OFF);
	 if(sensor_values[4] > 200)
	   sv4 = 1;
     else
	   sv4 = 0;
     if(sensor_values[3] > 200)
       sv3 = 2;
     else
       sv3 = 0;

     if (sensor_values[2] > 200)
       sv2 = 4;
     else
       sv2 = 0;

    if (sensor_values[1] > 200)
       sv1 = 8;
    else
       sv1 = 0;

    if (sensor_values[0] >200)
	   sv0 = 16;
    else
	   sv0 = 0;
    svtotal = sv4+sv3+sv2+sv1+sv0;
	// if sensor values have changed... stop motors
	if (oldIR != svtotal)
	  {
	   set_motors(0,0);  
	  }
    oldIR = svtotal;  // remember current sensors
	SensorBuffer[1]=svtotal; // prepare to send sensors back

	 // now read the bumper sensors	 
	 PORTD |= (1<<MuxCtl);  // select bumper sensors
     read_line_sensors_calibrated(sensor_values,IR_EMITTERS_OFF);
	 if(sensor_values[3] > 200)
	   sv4 = 1;
	 else
	   sv4 = 0;
	    
     if(sensor_values[3] > 200)
       sv3 = 2;
     else
       sv3 = 0;

     if (sensor_values[2] > 200)
       sv2 = 4;
     else
       sv2 = 0;

    if (sensor_values[1] > 200)
       sv1 = 8;
    else
       sv1 = 0;

    svtotal = sv4+sv3+sv2+sv1;
	// if sensor values have changed... stop motors
	if (oldBumpers != svtotal)
	  {
	   set_motors(0,0);  
	  }
    oldBumpers = svtotal;  // remember current sensors
	SensorBuffer[0]=svtotal; // prepare to send sensors back
     
     // send back critical sensor data
     serial_send_blocking(SensorBuffer,5);
}


// ===================================================================
// This is the main function, where the code starts.
// ===================================================================
int main()
{
	//unsigned int sensors[5]; // an array to hold sensor values
    int i;
	// set up the 3pi
	initialize();
    

	while (1)
      BlueToothControl();
}

/* ====================================================================
 *  The following routines are generic routines for reading I2C devices
 *  In this case, the only routines included are those specifically
 *  needed for reading and calibrating the HMC6352 compass.
 * ====================================================================
 */

void DataToInput(void)
  {
  DDRD &=  ~(1<<CompassData);
  }

void DataToOutput(void)
  {
  DDRD |= (1<<CompassData);
  }

void ClockToInput(void)
  {
  DDRD &=  ~(1<<CompassClock);
  }

void ClockToOutput(void)
  {
  DDRD |= (1<<CompassClock);
  }

void I2Cstart(void)
  {
  DataToOutput();
  ClockToOutput();
  }

void I2Cstop(void)
  {
  DataToOutput();
  ClockToInput();
  DataToInput();
  }

void I2Cack(void)
  {
  DataToOutput();
  ClockToInput();
  ClockToOutput();
  DataToInput();
  }

void ICnack(void)
  {
  DataToInput();
  ClockToInput();
  ClockToOutput();
  }

int I2Cread(void)
  {
  int i,temp=0;
  DataToInput();
  for(i=0;i<=7;i++)
    {
	temp=temp<<1;
    ClockToInput();
	if (PIND & (1<<CompassData))
	  temp=temp+1;
    ClockToOutput();
	}
  return temp;
  }

void I2Cwrite(int temp)
  {
  int i;
  for(i=0;i<=7;i++)
    {
	if (temp&128)
	  DataToInput();
    else
	  DataToOutput();
    ClockToInput();
	ClockToOutput();
	}
  DataToInput();
  ClockToInput();    // set data high
  if(!(PIND & (1<<CompassData))) // write last bit as the ACK bit
    DataToOutput();  // set data low if last bit is zero
  ClockToOutput();    // then clock it 
  }

int ReadCompass(void)
  {
  I2Cstart();
  I2Cwrite(0x42);
  I2Cwrite(0x41);
  I2Cstop();
  I2Cstart();
  I2Cwrite(0x43);
  DataToInput();
  }

void StartCalibration(void)
  {
  I2Cstart();
  I2Cwrite(0x42);
  I2Cwrite(0x43);
  I2Cstop();
  delay_us(10);
  }

void StopCalibration(void)
  {
  I2Cstart();
  I2Cwrite(0x42);
  I2Cwrite(0x45);
  delay_ms(15);
  }

void CalibrateCompass(void)
  {
  set_motors(0,0); // make sure motors are off
  delay_ms(300);
  //StartCalibration();
  set_motors(-20,+20);
  delay_ms(20);
  set_motors(0,0);
// ensure there is slightly more than 2 full turns
  //StopCalibration();
  }












/*
//------------------- stuff to be moved elsewhere
	// Display battery voltage and wait for button press
	while(!button_is_pressed(BUTTON_B))
	{
		int bat = read_battery_millivolts();

		clear();
		print_long(bat);
		print("mV");
		lcd_goto_xy(0,1);
		print("Press B5");

		delay_ms(100);
	}


	// Always wait for the button to be released so that 3pi doesn't
	// start moving until your hand is away from it.
	wait_for_button_release(BUTTON_B);
	delay_ms(1000);

	// Auto-calibration: turn right and left while calibrating the
	// sensors.
	for(counter=0;counter<80;counter++)
	{
		if(counter < 20 || counter >= 60)
			set_motors(40,-40);
		else
			set_motors(-40,40);

		// This function records a set of sensor readings and keeps
		// track of the minimum and maximum values encountered.  The
		// IR_EMITTERS_ON argument means that the IR LEDs will be
		// turned on during the reading, which is usually what you
		// want.
		calibrate_line_sensors(IR_EMITTERS_ON);

		// Since our counter runs to 80, the total delay will be
		// 80*20 = 1600 ms.
		delay_ms(20);
	 }


	// Display calibrated values as a bar graph.
	while(!button_is_pressed(BUTTON_B))
	{
		// Read the sensor values and get the position measurement.
		unsigned int position = read_line(sensors,IR_EMITTERS_ON);

// creating RobotBASIC sensor_value array
		unsigned int sensor_values[5];   
// Getting calibrated values and setting threshold to 200.  And then converting
// to RobotBASIC 0,1 digital value.  Then summing the 3 values for control logic.

read_line_sensors_calibrated(sensor_values,IR_EMITTERS_ON);

// creating RobotBASIC individual sensor variables from the middle three 3Pi sensors
// only using the middle 3 sensors of the 3Pi robot to match RobotBASIC
unsigned int sv3;  //light sensor 3  (pc3)
unsigned int sv2;  // light sensor 2  (pc2)
unsigned int sv1;  // light sensor 1 (pc1)
unsigned int svtotal;

if (sensor_values[3] > 200)
sv3 = 1;
else
sv3 = 0;

if (sensor_values[2] > 200)
sv2 = 2;
else
sv2 = 0;

if (sensor_values[1] > 200)
sv1 = 4;
else
sv1 = 0;

svtotal = sv3+sv2+sv1;


		// Display the position measurement, which will go from 0
		// (when the leftmost sensor is over the line) to 4000 (when
		// the rightmost sensor is over the line) on the 3pi, along
		// with a bar graph of the sensor readings.  This allows you
		// to make sure the robot is ready to go.
		clear();
		//print_long(position);  // old Pololu code
		//print_long(sensor_values[3]);  // old Pololu code
		print_long(svtotal);
		lcd_goto_xy(4,0);
		print_long(sensor_values[1]);
		lcd_goto_xy(0,1);
		//display_readings(sensors);  // old Pololu code
		display_readings(sensor_values);

		delay_ms(100);
	}
	wait_for_button_release(BUTTON_B);

	clear();

	print("RoBASIC");		

	// Play music and wait for it to finish before we start driving.
play_from_program_space(go);
while(is_playing());

//----------------
*/
