


/*          3Pi Command Processor for RobotBASIC with full sensor support
 *         Copyright January 2010 by John Blankenship, all rights reserved       
 * 
 * This code allows the 3Pi to communicate with RobotBASIC using the built-in 
 * communication protocol.  The sensors supported include 5 line sensors, 
 * 4 bumper sensors, 5 infrared perimeter sensors, 1 beacon detector (recognizes
 * multiple beacons), electronic compass, battery level, and distance measurement
 * using servo controlled turret.
 *
 * Care was taken to allow the STANDARD Pololu 3Pi library routines to be used for all sensors.
 * General information about the hardware modifications can be found at www.RobotBASIC.com
 *
 * A DETAILED descriptions of the physical and electrical modifications (including both hardware 
 * schematics and full explanations of the embedded software  as well as a
 * number of interesting projects are provided in the book "Enhancing the Pololu 3Pi
 * with RobotBASIC" (available on Amazon.com).   Special commands have been added to provide 
 * special initialization and calibration capabilities.
 *
 * Note: Bumper sensors and Infared Perimeter sensors are multiplexed over the existing
 * line sensors, using the standard library routines.
 *
 * Suggested websites:
 *     http://www.RobotBASIC.com
 *     http://www.pololu.com
 *     http://forum.pololu.com
 *
 */

// The 3pi include file must be at the beginning of any program that
// uses the Pololu AVR library and 3pi.  (Change the paths to match your directories)
#include <C:\Users\John\Documents\AVR\Include\pololu\3pi.h>
//#include <string.h>

#include <avr/IO.h>

// This include file allows data to be stored in program space.  The
// ATmega168 has 16k of program space compared to 1k of RAM, so large
// pieces of static data should be stored in program space.
#include <avr/pgmspace.h>


//====================================================================
// initialization of global variables and constants
//====================================================================

// A couple of simple tunes, stored in program space.
  const char welcome[] PROGMEM = ">g32>>c32";
  const char go[] PROGMEM = "L16 cdegreg4";

// New I/O pin definitions: Original standard 3Pi definitions still apply
  #define Servo PORTB0
  #define Beacon PORTB1
  #define CompassClock PORTD2
  #define CompassData PORTD4
  #define MuxCtl 7
  #define IRctl 5
  #define BUMPctl 4
  #define ServoCtr 1250

// some function prototypes
  void Calibrate_Compass(int);
  int  Read_Compass(void);



// global variables for communication buffers
  unsigned char incoming[2]={0,0};
  static char SensorBuffer[5]={0,0,0,0,0};


// ================================================================
// Initialization routine
// Initializes the 3pi, calibrates, and plays the initial music.
// ================================================================
void initialize()
{
	// This pololu routine must be called at the beginning of 3pi code, to set up the
	// sensors.  Pololu uses a value of 2000 for the timeout, which
	// corresponds to 2000*0.4 us = 0.8 ms on our 20 MHz processor.
	pololu_3pi_init(2000);
	
    // set up Direction Register when necessary for new pin assignments
       DDRB |= (1<<Servo);          // set pin to output for servo control
       DDRB &= ~(1<<Beacon);      // set pin to input to read beacon data
       DDRD |= 1<<MuxCtl;          // set pin to output to control MUX circuits

    // initially set Servo and Mux outputs LOW
	   PORTB &= ~(1<Servo);
	   PORTD &= ~(1<MuxCtl);

    // Compass I/O setup is special as the compass is I2C so both ports must float
    // Initial data is set LOW so any time the PIN is an OUTPUT, it will be LOW
    // When a PIN floats, EXTERNAL pull-up's allow it to appear HIGH
	   DDRD |=  CompassClock;        // make both pins output
	   DDRD |=  CompassData;         // so we can set the DATA register
	   PORTD &= ~(1<<CompassClock);   // Set both outputs LOW
	   PORTD &= ~(1<<CompassData);
	   DDRD &=  ~(1<<CompassClock);   // Set both to INPUT so they FLOAT
	   DDRD &=  ~(1<<CompassData);

       DDRB  |= (1<<4);  // make output pin  MUX control
       DDRB  |= (1<<5);  // make output pin


    // Note: The analog pins for reading the battery voltage and Distance value
	//       are already setup properly, as is PC5 which controls the infrared
	//       source for the line sensors (which must be turned OFF when reading
	//       the bumpers or the infrared parimeter sensors.  PD0 and PD1 are also
	//       properly set up for serial communication by the standard 3Pi initialization


	   PORTB |= (1<<4);  // low to disable extended sensors

       // do a simple sensor calibration... only enables IR and BUMPERS
			  calibrate_line_sensors(IR_EMITTERS_ON_AND_OFF);
              delay_ms(2000);
	   // Make OFF calibration full scale

              unsigned int* MINON = get_line_sensors_calibrated_minimum_on();
              unsigned int* MINOFF = get_line_sensors_calibrated_minimum_off();
	          unsigned int* MAXON = get_line_sensors_calibrated_maximum_on();
	          unsigned int* MAXOFF = get_line_sensors_calibrated_minimum_off();
              for (int j=0;j<5;j++)
		        {
		        *(MINOFF+j)=0;  //*(MINON+j);
		        *(MAXOFF+j)=1023; //*(MAXON+j);
		        }
              

	// Play welcome music
	play_from_program_space(welcome);
	delay_ms(1000);

    set_motors(0,0); // make sure motors are initially off
    serial_set_baud_rate(9600);
    delay_ms(200);
}

void MoveToAngle (int angle, int speed)
{
int i,j,temp;
if(angle<0) angle=angle+360;
for (i=0;i<=4;i++)
  {
  temp=angle;
  if (temp<Read_Compass()) temp=temp+360; //make destination larger
  if ((temp-Read_Compass())<180) // decide which direction to turn
    {
    set_motors(50,-50);// overcome friction etc
    delay_ms(5);
    set_motors(speed-(i*4)-6,-(speed-(i*4)-6)); // turn right
    }
  else
    {
    set_motors(-50,50);// overcome friction etc
    delay_ms(5);
    set_motors(-(speed-(i*4)-6),speed-(i*4)-6); // turn left
    }
  if(i==0)
    while ((Read_Compass()/32)!=(angle/32));
  if(i==1)
    while ((Read_Compass()/16)!=(angle/16));
  if(i==2)
    while ((Read_Compass()/8)!=(angle/8));
  if(i==3)
    while ((Read_Compass()/4)!=(angle/4));
  if(i==4)
    for(j=0;j<6;j++)
	   {
	   if ((Read_Compass()/2)==(angle/2)) break;
	   };
  set_motors(0,0);
  delay_ms(50);
  }
}


// ===================================================================
// Simple Bluetooth control module
// Assumes only one Bluetooth device is within range and installed
// Waits for commands from PC and returns sensor values as originally
// described in Robot Programmer's Bonanza by Blankenship/Mishal
// ===================================================================
void BlueToothControl(void)
{
   unsigned int         sensor_values[5];
   unsigned char        sv4;
   unsigned char        sv3;
   unsigned char        sv2;
   unsigned char        sv1;
   unsigned char        sv0;
   unsigned char        svtotal;
   static unsigned char oldSensors=0;
   static unsigned char command;
   static unsigned char argument;
   static char          ServoOffset=0;
   static int           OldServoAngle=0;
   static char          DistMode=0;
   static char          CompassEnabled = 0;     // Compass used by default
   static char          LineSensorsEnabled = 0; // Line sensors off by default
   int                  temp;
   int                  comp,temp1,temp2,temp3;
   static char          speed=40; 
   static char          Loffset=0;
   static char          Roffset=0;
   static unsigned char RotTime=12;
   static unsigned char MoveTime=24;
   static unsigned char TurnStyle=0;
   int                  i,battery,counter;
   int                  MyTimeOut=500;
   unsigned int         sum;
   unsigned int         ave;
   unsigned char        samples;
   static unsigned int  HiLevel=200;
   static unsigned char TranslateDist[32]={255,255,255,240,192,155,128,104,88,75,68,60,52,46,42,38,
                                            35, 32, 29, 26, 24, 22, 20, 18,16,14,12,11,10, 9, 8, 0};

   incoming[0]=0xff;    // prepare to check for timeout
   incoming[1]=0xff;
   serial_receive_blocking(incoming,2,MyTimeOut); // default 500ms
   command=incoming[0];
   argument=incoming[1];
   if ((command==0xff) || (argument==0xff))  // timeout occurred before data received
     {
     set_motors(0,0);
     return;
     }
   SensorBuffer[0]=0;
   SensorBuffer[1]=0;
   SensorBuffer[2]=0;
   SensorBuffer[3]=0;
   SensorBuffer[4]=0;
   switch (command)
   {     
          // ========================
          // ======= Locate =========
		  // ========================

		  case 3:
            set_motors(0,0);
			DDRB |= (1<<Servo);          // set pin to output for servo control
              for(i=0;i<30;i++) // do enough pulses to ensure movement
			    {
			    PORTB |= (1<<Servo);  // pulse line high
			    delay_us(ServoCtr+ServoOffset);
			    PORTB &= ~(1<<Servo); // return to low
			    delay_us(20000-ServoCtr);
				}
			  OldServoAngle=0;
			  delay_ms(50);
			play_from_program_space(welcome);
			delay_ms(400);
		  break;

          // ========================
          // ====== Forward =========
		  // ========================

		  case 6:
		    if (argument==1)
			   set_motors(speed+Loffset,speed+Roffset);
            if (argument==0)
			   set_motors(0,0);
        	if (argument>1)
			   {
			   if ((argument<50) || !CompassEnabled)
			      {
			      set_motors(speed+Loffset,speed+Roffset);
			      delay_ms(MoveTime*argument);
			      set_motors(0,0);
                  }
               else
                  {
				  temp=Read_Compass();
				  temp1=0;
				  for(i=0;i<((argument/2)*MoveTime)/48;i++)
				     {
                     set_motors(speed+Loffset,speed+Roffset+temp1);
					 comp=Read_Compass();
                     temp2=abs(temp-comp);
					 temp3=temp2;
					 if (temp2>50)
					   temp3 = 360-temp2;
					 if(temp!=comp)
					   {
					   if( ((comp<temp)&&(temp2<50)) || ((comp>temp)&&(temp2>50)) )
						 temp1=-(temp3);
                       else
						 temp1=temp3;
					   }
					 }
				  }
			   }	 			
		  break;

          // ========================
          // ====== Reverse =========
		  // ========================

		  case 7:
		    if (argument==1)
			  set_motors(-(speed+Loffset),-(speed+Roffset));
            if (argument==0)
			  set_motors(0,0);
        	if (argument>1)
			   {
			   set_motors(-(speed+Loffset),-(speed+Roffset));
			   delay_ms(MoveTime*argument);
			   set_motors(0,0);
			   }
		  break;

          // ========================
          // ======= Right  =========
		  // ========================

		  case 12:
		    if (argument==1)
			  {  
			  if (TurnStyle==0)
			     set_motors(speed/2,-speed/2);
			  if (TurnStyle>=1)
			     set_motors(speed+Loffset,(TurnStyle*(speed+Roffset))/10);
              }
            if (argument==0)
			  set_motors(0,0);
        	if (argument>1)
			   {
			   if ((argument<20) || !CompassEnabled)
			      { 
			      set_motors((speed*2)/3,-(speed*2)/3);
			      delay_ms(RotTime*argument);
			      set_motors(0,0);
				  }
			   else
			      {
			      temp=Read_Compass();
				  temp+=argument;
				  if (temp>=360) temp-=360;
				  MoveToAngle(temp,speed);
				  }
			   }
		  break;

          // ========================
          // ======= Left ===========
		  // ========================

		  case 13:
		    if (argument==1)
			  {
			  if (TurnStyle==0)
			     {
                 //set_motors(-speed,speed);// overcome friction etc
                 //delay_ms(2);
			     set_motors(-speed/2,speed/2);
				 }
			  if (TurnStyle>=1)
			     set_motors((TurnStyle*(speed+Loffset))/10,speed+Roffset);
              }
            if (argument==0)
		  	  {
			  set_motors(0,0);
              }
        	if (argument>1)
			   {  
			   if ((argument<20) || !CompassEnabled)
			      {
			      set_motors(-(speed*2)/3,(speed*2)/3);
			      delay_ms(RotTime*argument);
			      set_motors(0,0);
				  }
               else
			      {
				  temp=Read_Compass();
				  temp-=argument;
				  if (temp<0) temp+=360;
				  MoveToAngle(temp,speed);
				  }
			   }
		  break;

          // ========================
		  // ======= Speed ==========
		  // ========================

		  case 36:
		    // ignore rSpeed commands
		  break;

          // ========================
		  // ======= Compass ========
		  // ========================

		  case 24:
		    sum=Read_Compass();
            SensorBuffer[3]=sum>>8;
			SensorBuffer[4]=sum&255;
		  break;

          // ========================
		  // ======= Beacon =========
		  // ========================

		  case 96:
		    if (PINB&(1<<Beacon))   // it it happens to be High now
			  delay_ms(2);           // wait and try again
		    if (!(PINB&(1<<Beacon)))
			  {
			  if (argument>0)
			    {
			    // Beacon detected, now see which one
			    // first wait till beacon signal stops
			    while (!(PINB&(1<<Beacon)));
			    // then count the time till it starts again
			    delay_us(50);
			    for (i=1;i<=15;i++)
			      {
				  delay_us(100);
				  if (!(PINB&(1<<Beacon)))
				    break;
				  }
                if (i==argument)
			      {
			      SensorBuffer[3]=0;
			      SensorBuffer[4]=1; // true (found it)
				  }
                }
              else
			    {
				SensorBuffer[3]=0;
			    SensorBuffer[4]=1; // return True for ANY beacon 
				}
			  }
		  break;

          // ================================
		  // === Ranging Right (Distance) ===
		  // ================================

		  case 192:
		    if (OldServoAngle!=argument)
		      {
		      DDRB |= (1<<Servo);          // set pin to output for servo control
              for(i=0;i<30;i++) // do enough pulses to ensure movement
			    {
			    PORTB |= (1<<Servo);  // pulse line high
				temp = ServoCtr-(10*argument)+ServoOffset;
			    delay_us(temp);
			    PORTB &= ~(1<<Servo); // return to low
			    delay_us(20000-temp);
				}
			  OldServoAngle=argument;
			  delay_ms(20);
			  }
            // now take reading
            set_analog_mode(MODE_8_BIT); // 8-bit analog-to-digital conversions
            sum = 0;
            ave = 0;     
            for(samples=0;samples<10;samples++)
              {
			  start_analog_conversion(TRIMPOT); // start conversion
              while (analog_is_converting())
			     ; // wait till conversion is done...
              sum += analog_conversion_result(); // get result
              }
            ave = (sum / 10); // compute 10-sample average of ADC result
			SensorBuffer[3]=0;
			if (DistMode==2)  // raw
			  SensorBuffer[4]=ave;
            if (DistMode==1)  // inches
			  {
			  ave=ave/5;  
			  if (ave>31)
			    ave=31;
              SensorBuffer[4]=TranslateDist[ave]/8;
              }
            if (DistMode==0) // pixels
			  {
			  ave=ave/5;
			  if (ave>31)
			    ave=31;
              SensorBuffer[4]=TranslateDist[ave];
              }
		  break;

          // ================================
		  // === Ranging  Left (Distance) ===
		  // ================================

		  case 193:
		    if (-OldServoAngle!=argument)
		      {
		      DDRB |= (1<<Servo);          // set pin to output for servo control
              for(i=0;i<30;i++) // do enough pulses to ensure movement
			    {
			    PORTB |= (1<<Servo);  // pulse line high
				temp = ServoCtr+(argument*10)+ServoOffset;
			    delay_us(temp);
			    PORTB &= ~(1<<Servo); // return to low
			    delay_us(20000-temp);
				}
			  OldServoAngle=-argument;
			  delay_ms(20);
			  }
            // now take reading
            set_analog_mode(MODE_8_BIT); // 8-bit analog-to-digital conversions
            sum = 0;
            ave = 0;     
            for(samples=0;samples<10;samples++)
              {
			  start_analog_conversion(TRIMPOT); // start conversion
              while (analog_is_converting())
			     ; // wait till conversion is done...
              sum += analog_conversion_result(); // get result
              }
            ave = (sum / 10); // compute 10-sample average of ADC result
            SensorBuffer[3]=0;
			if (DistMode==2)  // raw
			  SensorBuffer[4]=ave;
            if (DistMode==1)  // inches
			  {
			  ave=ave/5;
			  if (ave>31)
			    ave=31;
              SensorBuffer[4]=TranslateDist[ave]/8;
              }
            if (DistMode==0)  // pixels
			  {
			  ave=ave/5;
			  if (ave>31)
			    ave=31;
              SensorBuffer[4]=TranslateDist[ave];
              }
		  break;

          // ================================
		  // ======== Charge Level ==========
		  // ================================
		  case 108:
		    battery=read_battery_millivolts();
			battery=battery/52;
			if (battery>100) battery=100;
			SensorBuffer[3]=battery>>8;
			SensorBuffer[4]=battery&255;
		  break;

          // The following commands are issued from RobotBASIC through the rCommand statement

		  // ================================
		  // ======= Set 3Pi Time Out =======
		  // ================================
		  case 110:
		     MyTimeOut=argument;
          break;
		 
		  // =========================
		  // ======= Play Sound ======
		  // =========================
		  case 111:
		    if (argument==1) play_from_program_space(welcome);
			if (argument==2) play_from_program_space(go);
			if (argument==3) play("c");
			if (argument==4) play("cdefedc");
		  break;

          // ================================
		  // ======= Calibrate Servo ========
		  // ================================
		  case 112:
		    ServoOffset=argument;
		    if (ServoOffset>127)
		      ServoOffset-=256;
		  break;

          // ================================
		  // ==== Calibrate Line Sensors ====
		  // ================================
		  case 113:
		    PORTB |= (1<<4);  // line high to disable IR and bumper
		    set_motors(0,0);
		    delay_ms(100);
     	    // Auto-calibration: turn right and left while calibrating the sensors
	        for(counter=0;counter<80;counter++)
	          {
		      if(counter < 20 || counter >= 60)
			     set_motors(speed-10,-(speed-10));
		      else
			     set_motors(-(speed-10),speed-10);

		      // This function records a set of sensor readings and keeps
		      // track of the minimum and maximum values encountered.  The
		      // IR_EMITTERS_ON argument means that the IR LEDs will be
		      // turned on during the reading, which is usually what you want
		      calibrate_line_sensors(IR_EMITTERS_ON_AND_OFF);

		      // Since our counter runs to 80, the total delay will be
		      // 80*20 = 1600 ms.
		      delay_ms(20);
	          }
			  set_motors(0,0);
			  		// Make OFF calibration Same as ON calibration

              unsigned int* MINON = get_line_sensors_calibrated_minimum_on();
              unsigned int* MINOFF = get_line_sensors_calibrated_minimum_off();
	          unsigned int* MAXON = get_line_sensors_calibrated_maximum_on();
	          unsigned int* MAXOFF = get_line_sensors_calibrated_minimum_off();
              for (int j=0;j<5;j++)
		        {
		        *(MINOFF+j)=0;    //*(MINON+j);
		        *(MAXOFF+j)=1023; //*(MAXON+j);
		        }
              LineSensorsEnabled = 1;
		  break;

          // ================================
		  // ==== Calibrate Rotation Time ===
		  // ================================
		  case 114:
		    RotTime=argument;
		  break;

          // ================================
		  // ====== Calibrate F/R Time ======
		  // ================================
		  case 115:
		    MoveTime=argument;
		  break;

          // ================================
		  // ======= Calibrate Drift ========
		  // ================================
		  case 116:
		    Roffset = argument&15;
			Loffset = argument>>4;
			if (Roffset>7) Roffset=16-Roffset;
			if (Loffset>7) Loffset=16-Loffset;
		  break;

          // ================================
		  // ====== Calibrate Compass =======
		  // ================================
		  case 117:
		    Calibrate_Compass((int)speed);
			CompassEnabled = 1;
		  break;

          // ================================
		  // ====== Style of Turns    =======
		  // ================================
		  case 118:
		    TurnStyle=argument;
			if (TurnStyle>8) TurnStyle=8;
		  break;

          // ================================
		  // ========== Halt ================
		  // ================================
		  case 119:
		    set_motors(0,0);
          break;

          // ================================
		  // ======== Reset Defaults ========
		  // ================================
		  case 120:
		    speed=40;
			RotTime=12;
			MoveTime=24;
			Loffset=0;
			Roffset=0;
			TurnStyle=0;
			ServoOffset=0;
			DistMode=0;
			//NumBeacons=2;
			HiLevel=200;
			CompassEnabled=0;
			LineSensorsEnabled=0;
			MyTimeOut=500;
		  break;

		  // ================================
		  // ====== Enable Compass ==========
		  // ================================
		  case 121: 
            CompassEnabled=argument;
		  break;

		  case 122:
            // ================================
		    // ==== Set Sensor High Level  ====
		    // ================================
		     HiLevel=2*argument;
          break;

		  // ================================
		  // ====== Set Distance Mode =======
		  // ================================
		  case 123:
		    DistMode=argument;
          break;

  
		  case 124:
		  // ================================
		  // ======== Find a Beacon =========
		  // ================================
		     // argument specifies direction
			   temp1=0;
			   if (argument==1)
			     {
		         set_motors( speed,-speed ); // turn right
				 }
               else
			     {
		         set_motors(-speed,speed); // turn left
				 }
			   while(!(PINB&(1<<Beacon)))  // move away from current beacon
			      ;
               delay_ms(200);
               for(temp=0;temp<2500;temp++) // enough time for at least one FULL revolution
			     {
				 delay_ms(1);
                 if(!(PINB&(1<<Beacon)))
				   {
				   temp1=1;
				   break;  // look for new beacon
				   }
				 }
               set_motors(0,0);
			   SensorBuffer[4]=temp1;
		  break;

		  case 125:
		  // ================================
		  // == Turn To Specified Degree/2 ==
		  // ================================
		     MoveToAngle(argument*2,speed);
		  break;

		  case 126:
          // ================================
		  // ====   Set Robot's Speed    ====
		  // ================================
		     speed=argument;
		  break;		  
	 };

  // now check line sensors
  if (LineSensorsEnabled)
     {
	 // now read the line sensors
	 PORTB |= (1<<4);  // line high to disable IR and bumpers
     delay_ms(3);
     read_line_sensors_calibrated(sensor_values,IR_EMITTERS_ON);
	 if(sensor_values[4] > HiLevel)
	   sv4 = 8;
     else
	   sv4 = 0;
     if(sensor_values[3] > HiLevel)
       sv3 = 1;
     else
       sv3 = 0;

     if (sensor_values[2] > HiLevel)
       sv2 = 2;
     else
       sv2 = 0;

     if (sensor_values[1] > HiLevel)
       sv1 = 4;
     else
       sv1 = 0;

     if (sensor_values[0] > HiLevel)
	   sv0 = 16;
     else
	   sv0 = 0;

     svtotal = sv4+sv3+sv2+sv1+sv0;
     oldSensors = svtotal;  // remember current sensors
  	 SensorBuffer[2] = svtotal; // prepare to send sensors back
     }


  // now read the IR sensors	 

	 PORTC &= ~(1<<5);  // off lower lights 
	 PORTB &= ~(1<<IRctl);  // set low to enable IR sensors
	 PORTB &= ~(1<<4);      // enable extended sensors
	 delay_ms(5);
     read_line_sensors_calibrated(sensor_values,IR_EMITTERS_OFF);

     PORTB |= (1<<4); // disable extended sensors

	 if(sensor_values[4] > HiLevel)
	   sv4 = 1;
     else
	   sv4 = 0;
     if(sensor_values[3] > HiLevel)
       sv3 = 2;
     else
       sv3 = 0;

     if (sensor_values[2] > HiLevel)
       sv2 = 4;
     else
       sv2 = 0;

     if (sensor_values[1] > HiLevel)
       sv1 = 8;
     else
       sv1 = 0;

     if (sensor_values[0] > HiLevel)
	   sv0 = 16;
     else
	   sv0 = 0;
     svtotal = sv4+sv3+sv2+sv1+sv0;
	 SensorBuffer[1]=svtotal;
     

  // now read the bumper sensors	 

	 PORTC &= ~(1<<IRctl);  // off lower lights	 
	 PORTB |= (1<<IRctl);  // lines high to enable bumpers
	 PORTB &= ~(1<<4);      // enable extended sensors	 
	 delay_ms(1);

     read_line_sensors_calibrated(sensor_values,IR_EMITTERS_OFF);
	 

     //PORTB &= ~(1<<IRctl);  //set low to disable bumpers

	 if(sensor_values[4] > HiLevel)
	   sv4 = 1;
	 else
	   sv4 = 0;
	    
     if(sensor_values[3] > HiLevel)
       sv3 = 2;
     else
       sv3 = 0;

     if (sensor_values[2] > HiLevel)
       sv2 = 4;
     else
       sv2 = 0;

    if (sensor_values[1] > HiLevel)
       sv1 = 8;
    else
       sv1 = 0;

    svtotal = sv4+sv3+sv2+sv1;
	SensorBuffer[0]= svtotal; // prepare to send sensors back 
     
     // send back critical sensor data
	 // note: if any CASE needs to send 5 bytes, that case
	 // should provide the SEND (as below) and a RETURN

     serial_send_blocking(SensorBuffer,5);
}
  

// ===================================================================
// This is the main function, where the code starts.
// ===================================================================
int main()
{
    int i;
	// set up the 3pi
	initialize();
	while (1)
      BlueToothControl();
}

/* ====================================================================
 *  The following routines are generic routines for reading I2C devices
 *  In this case, the only routines included are those specifically
 *  needed for reading and calibrating the HMC6352 compass.
 * ====================================================================
 */

#define  SDA_PIN          PORTD2       //compass data pin
#define  SCL_PIN          PORTD4       //compas clockin  pin

#define  HMC6352_READ     0x43         //HMC6352 read address
#define  HMC6352_WRITE    0x42         //HMC6352 write address

#define  START_CALIB      0x43         //Start the calibration routine
#define  END_CALIB        0x45         //Finish the calibration routine
#define  GET_HEADING      0x41         //Read in the calculated heading


#define  GET_HEADING_DELAY  6          //6   ms
#define  START_CALIB_DELAY  10         //10  s
#define  END_CALIB_DELAY    1400       //14  ms

//------these may not be needed--------------------------
#define  ENTER_SLEEP      0x53         //Enter sleep mode
#define  WAKE_UP          0x57         //Exit sleep mode                                                                
#define  MODE_ADDRESS     0x4E         //RAM/EEPROM address for operational mode
#define  UPDATE           0x4F         //
#define  SAVE_MODE        0x4C         //Write the operational mode from RAM to EEPROM
#define  WRITE_EEPROM     0x77         //Read a byte from EEPROM
#define  READ_EEPROM      0x72         //Write a byte to EEPROM
#define  WRITE_RAM        0x47         //Read a byte from RAM
#define  READ_RAM         0x67         //Write a byte to RAM 


#define  READ_WRITE_DELAY   70         //70  s
#define  ENTER_SLEEP_DELAY  10         //10  s
#define  WAKE_UP_DELAY      80         //80  s
#define  UPDATE_DELAY       6000       //6   ms
#define  SAVE_MODE_DELAY    125        //125 s
//--------------------------------------------------------

//========================================================
// support functions
//========================================================
//--------------------------------------------------------
void SetSDA_Input(void)
{
  DDRD &=  ~(1<<SDA_PIN);
  delay_us (500);
}
//--------------------------------------------------------
void SetSDA_Output(void)
{
  DDRD |= (1<<SDA_PIN);
  delay_us (500);
}
//--------------------------------------------------------
void SetSCL_Input(void)
{
  DDRD &=  ~(1<<SCL_PIN);
  delay_us (500);
}
//--------------------------------------------------------
void SetSCL_Output(void)
{
  DDRD |= (1<<SCL_PIN);
  delay_us (500);
}
//--------------------------------------------------------
//=================================================================================
//=================================================================================
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//   I2C Bit Banging Low Level Routines
//   the routines assume that the SDA and SCL lines have pull-up resistors
//   which will pull the line High when the SDA or SCL pins go floating (ie. inputs)
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//=================================================================================
//=================================================================================
void InitI2C(void)
{
    // Compass I/O setup is special as the compass is I2C so both ports must float
    // Initial data is set LOW so any time the PIN is an OUTPUT, it will be LOW
    // When a PIN floats, EXTERNAL pull-up's allow it to appear HIGH
	   DDRD |=  CompassClock;        // make both pins output
	   DDRD |=  CompassData;         // so we can set the DATA register
	   PORTD &= ~(1<<CompassClock);   // Set both outputs LOW
	   PORTD &= ~(1<<CompassData);
	   DDRD &=  ~(1<<CompassClock);   // Set both to INPUT so they FLOAT
	   DDRD &=  ~(1<<CompassData);
}

void I2C_Start(void)
{
  //I2C start condition.  SDA transitions from high to low while the clock is high.
  //Assumes SDA and SCL are both floating high 
  SetSDA_Output();  //Pull SDA low
  SetSCL_Output();  //Pull SCL low
}
//=================================================================================
void I2C_Stop(void)
{
  // I2C stop condition.  SDA transitions from low to high while the clock is high.
  // Assumes SCL is held low
  SetSDA_Output();     //Pull SDA low
  SetSCL_Input();      //Let SCL float high
  SetSDA_Input();      //Let SDA float high    
}   
//=================================================================================
void I2C_ACK(void)
{
  //Acknowledge for read; use between reading successive bytes
  SetSDA_Output();     //Pull SDA low
  SetSCL_Input();      //Clock out an acknowledge bit
  SetSCL_Output();     //
  SetSDA_Input();      //Let SDA float high    
}
//=================================================================================
void I2C_NACK(void)
{
  // Non-acknowledge for read; use after reading last byte
  SetSDA_Input();      //let SDA float high
  SetSCL_Input();      //Clock out an acknowledge bit
  SetSCL_Output();     //
}
//=================================================================================
int I2C_Read(void)
{
  //Read a byte from the I2C device I2C_ReadByte will hold the read byte
  int I2C_ReadByte = 0;
  int i;
  SetSDA_Input();                         //Let SDA float
  for( i=0; i<8;i++)
  {                                       //for 8 bits starting with the MSBit
     I2C_ReadByte = I2C_ReadByte << 1;    //shift last read bit to left
     SetSCL_Input();                      //let SCL float
     if(PIND &(1<<SDA_PIN))               //read port and mask for SDA pin
       I2C_ReadByte++;                    //if high then increment to set bit
     SetSCL_Output();                     //Pull SCL low
  }   
  return I2C_ReadByte;
}
//=================================================================================
int I2C_Write(int I2C_WriteByte)
{
  //Write a byte to the I2C device I2C_WriteByte is the byte 
  //to write and I2C_AckBit will hold the Ack from slave
  int i;
  for( i=7;i>=0;i--)
  {                                //for 8 bits starting at the MSB
    if (I2C_WriteByte&(1<<i))                //examine bit in the Data byte
      SetSDA_Input();                 //if high then set SDA to input ie floating => high
    else  
      SetSDA_Output();                //if low then make SDA output and thus low
    SetSCL_Input();                   //Pulse SCL high then low
    SetSCL_Output();
  }
  SetSDA_Input();                     //Let SDA float
  SetSCL_Input();                     //Start a pulse on SCL
  i = 0;
  if(PIND & (1<< SDA_PIN))            // read the SDL pin for the ack status
    i = 1;                            //set i to 1 if SDL is high
  else
    i = 0;  
  SetSCL_Output();                    //Finish a pulse on SCL
  return i; 
}
//=================================================================================
//========================================================
// High Level Compass functions
//========================================================
//--------------------------------------------------------
int Read_Compass(void)
{ // returns compass Heading
  int Heading;
  InitI2C();
  I2C_Start();
  I2C_Write(HMC6352_WRITE);
  I2C_Write(GET_HEADING);
  I2C_Stop();
   
  I2C_Start();
  I2C_Write(HMC6352_READ);

  delay_ms(12); //GET_HEADING_DELAY);
  Heading = I2C_Read() << 8;
  I2C_ACK();
  Heading = Heading |I2C_Read();
  I2C_NACK();
  I2C_Stop();
  return Heading/10;
}
//--------------------------------------------------------
void Start_Calibration(void)
{
  I2C_Start();
  I2C_Write(HMC6352_WRITE);
  I2C_Write(START_CALIB);
  I2C_Stop();
  delay_us(START_CALIB_DELAY);
}
//--------------------------------------------------------
void Finish_Calibration(void)
{
  I2C_Start();
  I2C_Write(HMC6352_WRITE);
  I2C_Write(END_CALIB);
  I2C_Stop();
  delay_us(END_CALIB_DELAY);
}
//--------------------------------------------------------
void Calibrate_Compass(int speed)
{
  // not yet finished
  set_motors(0,0); // make sure motors are off
  delay_ms(300);
  Start_Calibration();
  set_motors(-(speed*2)/3,(speed*2)/3);
  delay_ms(25000);
  set_motors(0,0);
  Finish_Calibration();
}
//--------------------------------------------------------



